import asyncio
import os
import sys

if sys.platform == 'win32':
    asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())

sys.path.append(os.path.abspath(os.path.join(os.path.abspath(sys.argv[0]), '..')))
sys.path.append(os.path.abspath(os.path.join(os.path.abspath(sys.argv[0]), '../..')))



import sys
import asyncio
import requests
import socket
from datetime import datetime, timedelta
from traceback import print_exc
from signal import signal, SIGINT, SIGTERM


async def main(spider, mode, sync, timer):
    try:
        rabbit = spider(sync)
    except Exception:
        print_exc()
        raise

    # def signal_handler(sig, frame):
    #     requests.post('http://127.0.0.1:8000/post/task',
    #                   json={'name': rabbit.name, 'status': 0,
    #                         'stop_time': datetime.now().strftime('%Y-%m-%d %H:%M:%S')})
    #
    # signal(SIGINT, signal_handler)
    # signal(SIGTERM, signal_handler)
    try:
        # requests.post('http://127.0.0.1:8000/post/task',
        #               json={'name': rabbit.name, 'ip_address': f'{socket.gethostbyname(socket.gethostname())}',
        #                     'sync': sync, 'status': 1})
        await rabbit.run(mode)
        # if timer:
        #     requests.post('http://127.0.0.1:8000/post/task',
        #                   json={'name': rabbit.name,
        #                         'sleep': timer,
        #                         'next_time': (datetime.now() + timedelta(minutes=timer)).strftime('%Y-%m-%d %H:%M:%S')})
        # elif mode == 'auto':
        #     requests.post('http://127.0.0.1:8000/delete/queue', json={'name': rabbit.name})
    except Exception:
        print_exc()


async def go(spider, mode: str = 'auto', sync: int = 1, timer: int = 0):
    for i in sys.argv[1:]:
        key, value = i.split('=')
        if key == 'mode':
            mode = value
        if key == 'sync':
            sync = value
    while timer:
        await main(spider, mode=mode, sync=sync, timer=timer)
        await asyncio.sleep(timer * 60)
    else:
        await main(spider, mode=mode, sync=sync, timer=timer)



import warnings
from typing import Callable, Optional, Union
from pydantic import BaseModel, field_validator

warnings.filterwarnings("ignore", category=UserWarning, module="pydantic")


class Request(BaseModel):
    url: Optional[str] = None
    params: Optional[dict] = None
    data: Union[dict, str, bytes, None] = None
    json: Optional[dict] = None
    method: Optional[str] = 'get'
    headers: Optional[dict] = None
    timeout: Optional[int] = None
    cookies: Optional[dict] = None
    proxy: Optional[str] = None
    allow_redirects: Optional[bool] = True
    callback: Union[Callable, str] = 'parse'
    retry: Optional[int] = 0
    meta: Optional[dict] = {}

    @field_validator('callback')
    def call(cls, key):
        if callable(key):
            return key.__name__
        else:
            return key



import os
import pickle
import hashlib
import sys
from io import BytesIO
import redis
import json
from RabbitSpider.dupefilters import DupeFilter
from urllib.parse import urlencode


class RedisDupeFilter(DupeFilter):
    def __init__(self, settings):
        super().__init__(settings)
        self.repeat = os.path.basename(sys.argv[0])
        self.redis = redis.StrictRedis(host=settings.get('REDIS_HOST'), port=settings.get('REDIS_PORT'),
                                       db=settings.get('REDIS_DB'), password=settings.get('REDIS_PASSWORD'))
        self.redis.delete(self.repeat)

    def request_seen(self, request):
        if isinstance(request.data, (dict, list, tuple)):
            body = urlencode(request.data).encode('utf-8')
        elif isinstance(request.data, str):
            body = request.data.encode('utf-8')
        elif isinstance(request.data, BytesIO):
            body = request.data.read()
        elif isinstance(request.data, bytes):
            body = request.data
        else:
            body = b""

        if request.json is not None:
            body = json.dumps(request.json, separators=(",", ":")).encode()

        sha1 = hashlib.sha1()
        if isinstance(request.params, (dict, list, tuple)):
            sha1.update(f'{request.url}?{urlencode(request.params)}'.encode('utf-8'))
        else:
            sha1.update(request.url.encode('utf-8'))
        sha1.update(request.method.encode('utf-8'))
        sha1.update(body)
        sha1.update(str(request.retry).encode('utf-8'))
        fingerprint = sha1.hexdigest()
        data = pickle.dumps(fingerprint)
        if self.redis:
            result = self.redis.sadd(self.repeat, data)
            return result



import os
import shutil
import argparse
from string import Template
from RabbitSpider.utils.control import SettingManager

settings = SettingManager({})


def tmpl_file_path(_path):
    for i in os.listdir(_path):
        if os.path.isfile(os.path.join(_path, i)):
            if i.endswith('tmpl'):
                yield os.path.join(_path, i)
        if os.path.isdir(os.path.join(_path, i)):
            for i in tmpl_file_path(os.path.join(_path, i)):
                yield i


def template_to_file(_path, **kwargs):
    if kwargs['project'].lower() == 'test':
        print(f'项目名称不可以是{kwargs["project"]}')
        return
    try:
        shutil.copytree(settings.get('TEMPLATE_DIR'), _path)
    except Exception:
        print(f'{_path}目录已存在')
    else:
        for file in tmpl_file_path(_path):
            with open(file, 'r', encoding='utf-8') as f:
                text = Template(f.read()).substitute(**kwargs)
            with open(file.replace('tmpl', 'py'), 'w', encoding='utf-8') as f:
                f.write(text)
            os.remove(file)
        os.rename(os.path.join(_path, 'spiders', 'basic.py'),
                  os.path.join(_path, 'spiders', f'{kwargs["spider"].lower()}.py'))
        print(f'{_path}创建完成')


def create_project():
    parser = argparse.ArgumentParser()
    parser.add_argument('create', default='create', help='参数：create')
    parser.add_argument('project', help='参数：项目名称')
    parser.add_argument('spider', help='参数：文件名称')
    args = parser.parse_args()
    template_to_file(f'./{args.project}', **{'project': f'{args.project}', 'spider': f'{args.spider.capitalize()}'})



from loguru import logger


class Logger(object):
    def __init__(self, settings, name):
        logger.add("../rabbit_log/rabbit_{time:YYYY-MM-DD}.log", level=settings.get('LOG_LEVEL'), rotation="1 day",
                   retention="1 week",
                   format="{time:YYYY-MM-DD HH:mm:ss} | {level} | {extra[scope]} | {name}:{line} - {message}")
        self.logger = logger.bind(scope=name)

    def log(self, msg):
        self.logger.info(msg)

    def warning(self, msg):
        self.logger.warning(msg)

    def error(self, msg):
        self.logger.error(msg)
